@@ -1,6 +1,22 @@
 /*
  * A basic toolset containing miscellaneous functions for string manipluation,
  * encoding/decoding, and a bunch of other stuff.
+ *
+ * Copyright (c) 1987-2011 by the citadel.org team
+ *
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 
@@ -524,9 +540,9 @@ size_t striplt(char *buf) {
  * \brief check for the presence of a character within a string (returns count)
  * \param st the string to examine
  * \param ch the char to search
- * \return the position inside of st
+ * \return the number of times ch appears in st
  */
-int haschar(const char *st,int ch)
+int haschar(const char *st, int ch)
 {
 	const char *ptr;
 	int b;
@@ -771,19 +787,30 @@ int stripout(char *str, char leftboundary, char rightboundary) {
  * Reduce a string down to a boundarized substring (for example, remove
  * parentheses and anything outside them).
  */
-void stripallbut(char *str, char leftboundary, char rightboundary) {
-	int a;
+long stripallbut(char *str, char leftboundary, char rightboundary) {
+	long len = 0;
 
-	for (a = 0; a < strlen(str); ++ a) {
-		if (str[a] == leftboundary) strcpy(str, &str[a+1]);
-	}
+	char *lb = NULL;
+	char *rb = NULL;
 
-	for (a = 0; a < strlen(str); ++ a) {
-		if (str[a] == rightboundary) str[a] = 0;
+	lb = strrchr(str, leftboundary);
+	if (lb != NULL) {
+		++lb;
+		rb = strchr(str, rightboundary);
+		if ((rb != NULL) && (rb >= lb))  {
+			*rb = 0;
+			fflush(stderr);
+			len = (long)rb - (long)lb;
+			memmove(str, lb, len);
+			str[len] = 0;
+			return(len);
+		}
 	}
 
+	return (long)strlen(str);
 }
 
+
 char *myfgets(char *s, int size, FILE *stream) {
 	char *ret = fgets(s, size, stream);
 	char *nl;
@@ -847,12 +874,20 @@ char *strcpy(char *dest, const char *src) {
  * Generate a new, globally unique UID parameter for a calendar etc. object
  */
 void generate_uuid(char *buf) {
-	static int seq = 0;
+	static int seq = (-1);
 
-	sprintf(buf, "%lx-%lx-%x",
-		time(NULL),
+	if (seq == (-1)) {
+		seq = (int)rand();
+	}
+	++seq;
+	seq = (seq % 0x0FFF) ;
+
+	sprintf(buf, "%08lx-%04lx-4%03x-a%03x-%012lx",
+		(long)time(NULL),
 		(long)getpid(),
-		(seq++)
+		seq,
+		seq,
+		(long)rand()
 	);
 }
 
